(function (global, factory) {
    if (typeof define === "function" && define.amd) {
        define(['exports', 'react', 'prop-types', './arrays', './get-outer-boundries'], factory);
    } else if (typeof exports !== "undefined") {
        factory(exports, require('react'), require('prop-types'), require('./arrays'), require('./get-outer-boundries'));
    } else {
        var mod = {
            exports: {}
        };
        factory(mod.exports, global.react, global.propTypes, global.arrays, global.getOuterBoundries);
        global.FocusOverlay = mod.exports;
    }
})(this, function (exports, _react, _propTypes, _arrays, _getOuterBoundries2) {
    'use strict';

    Object.defineProperty(exports, "__esModule", {
        value: true
    });
    exports.getOverlayStyles = undefined;

    var _react2 = _interopRequireDefault(_react);

    var _propTypes2 = _interopRequireDefault(_propTypes);

    var _getOuterBoundries3 = _interopRequireDefault(_getOuterBoundries2);

    function _interopRequireDefault(obj) {
        return obj && obj.__esModule ? obj : {
            default: obj
        };
    }

    function _classCallCheck(instance, Constructor) {
        if (!(instance instanceof Constructor)) {
            throw new TypeError("Cannot call a class as a function");
        }
    }

    var _createClass = function () {
        function defineProperties(target, props) {
            for (var i = 0; i < props.length; i++) {
                var descriptor = props[i];
                descriptor.enumerable = descriptor.enumerable || false;
                descriptor.configurable = true;
                if ("value" in descriptor) descriptor.writable = true;
                Object.defineProperty(target, descriptor.key, descriptor);
            }
        }

        return function (Constructor, protoProps, staticProps) {
            if (protoProps) defineProperties(Constructor.prototype, protoProps);
            if (staticProps) defineProperties(Constructor, staticProps);
            return Constructor;
        };
    }();

    function _possibleConstructorReturn(self, call) {
        if (!self) {
            throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
        }

        return call && (typeof call === "object" || typeof call === "function") ? call : self;
    }

    function _inherits(subClass, superClass) {
        if (typeof superClass !== "function" && superClass !== null) {
            throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
        }

        subClass.prototype = Object.create(superClass && superClass.prototype, {
            constructor: {
                value: subClass,
                enumerable: false,
                writable: true,
                configurable: true
            }
        });
        if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
    }

    var _extends = Object.assign || function (target) {
        for (var i = 1; i < arguments.length; i++) {
            var source = arguments[i];

            for (var key in source) {
                if (Object.prototype.hasOwnProperty.call(source, key)) {
                    target[key] = source[key];
                }
            }
        }

        return target;
    };

    var MAX_OPACITY = 0.8;
    var TRANSITION_SPEED = 0.3;

    var getOverlayStyles = exports.getOverlayStyles = function getOverlayStyles(x, y, width, height, animate) {
        var shared = {
            position: 'fixed',
            backgroundColor: '#000',
            transition: animate ? 'all ' + TRANSITION_SPEED + 's linear' : 'none'
        };

        var styles = {
            top: _extends({}, shared),
            bottom: _extends({}, shared),
            left: _extends({}, shared),
            right: _extends({}, shared),
            topleft: _extends({}, shared),
            topright: _extends({}, shared),
            bottomleft: _extends({}, shared),
            bottomright: _extends({}, shared)
        };
        if (typeof x === 'number') {
            //left column
            styles.topleft.left = styles.left.left = styles.bottomleft.left = '0';
            styles.topleft.width = styles.left.width = styles.bottomleft.width = x + 'px';

            //center column
            styles.top.left = styles.bottom.left = x + 'px';
            styles.top.width = styles.bottom.width = width + 'px';

            //right column
            styles.topright.left = styles.right.left = styles.bottomright.left = x + width + 'px';
            styles.topright.right = styles.right.right = styles.bottomright.right = '0';

            //top row
            styles.topleft.top = styles.top.top = styles.topright.top = '0';
            styles.topleft.height = styles.top.height = styles.topright.height = y + 'px';

            //center row
            styles.left.top = styles.right.top = y + 'px';
            styles.left.height = styles.right.height = height + 'px';

            //bottom row
            styles.bottomleft.top = styles.bottom.top = styles.bottomright.top = y + height + 'px', styles.bottomleft.bottom = styles.bottom.bottom = styles.bottomright.bottom = '0';
        }
        return styles;
    };

    var FocusOverlay = function (_React$Component) {
        _inherits(FocusOverlay, _React$Component);

        function FocusOverlay(props) {
            _classCallCheck(this, FocusOverlay);

            var _this = _possibleConstructorReturn(this, (FocusOverlay.__proto__ || Object.getPrototypeOf(FocusOverlay)).call(this, props));

            _this.state = {
                opacity: 0,
                visibility: 'hidden',
                animate: true
            };
            _this.onScroll = _this.onScroll.bind(_this);
            return _this;
        }

        _createClass(FocusOverlay, [{
            key: 'componentDidMount',
            value: function componentDidMount() {
                window.addEventListener('scroll', this.onScroll);
            }
        }, {
            key: 'componentWillUnmount',
            value: function componentWillUnmount() {
                window.removeEventListener('scroll', this.onScroll);
            }
        }, {
            key: 'onScroll',
            value: function onScroll() {
                var _this2 = this;

                this.setState({ animate: false });
                this.tryMove(this.props.focusComponent);
                if (this.timeoutReference) {
                    clearTimeout(this.timeoutReference);
                }
                this.timeoutReference = setTimeout(function () {
                    _this2.setState({ animate: true });
                }, 70);
            }
        }, {
            key: 'transitionStyles',
            value: function transitionStyles(opacity, x, y, width, height) {
                var _this3 = this;

                var newState = void 0;

                if (x) {
                    //if any coordinates have been passed then try set all the coordinates
                    newState = { x: x, y: y, width: width, height: height, opacity: opacity };
                } else {
                    //otherwise ignore all but opacity.
                    newState = { opacity: opacity };
                }

                if (opacity === 0) {
                    //if the target state is hidden, the we need to set the visibility to hidden after the opacity transition
                    setTimeout(function () {
                        _this3.setState({
                            visibility: 'hidden',
                            x: undefined, y: undefined, width: undefined, height: undefined
                        });
                    }, TRANSITION_SPEED * 1000);
                } else {
                    newState.visibility = 'visible';
                }
                setTimeout(function () {
                    _this3.setState(newState);
                });
            }
        }, {
            key: 'tryMove',
            value: function tryMove(focusComponent) {
                if (!focusComponent || (0, _arrays.isEmptyArray)(focusComponent)) {
                    if (this.state.opacity !== 0) {
                        this.transitionStyles(0);
                    }
                } else {
                    var _getOuterBoundries = (0, _getOuterBoundries3.default)(focusComponent),
                        x = _getOuterBoundries.x,
                        y = _getOuterBoundries.y,
                        width = _getOuterBoundries.width,
                        height = _getOuterBoundries.height;

                    if (this.state.x !== x || this.state.y !== y || this.state.width !== width || this.state.height !== height || this.state.opacity !== MAX_OPACITY) {
                        this.transitionStyles(MAX_OPACITY, x, y, width, height);
                    }
                }
            }
        }, {
            key: 'renderChildren',
            value: function renderChildren(currentContext, children, target) {
                if (target == currentContext) {
                    return children;
                }
                return null;
            }
        }, {
            key: 'render',
            value: function render() {
                var _props = this.props,
                    focusComponent = _props.focusComponent,
                    onExit = _props.onExit,
                    children = _props.children,
                    target = _props.target;

                var styles = void 0;
                this.tryMove(focusComponent);

                var _state = this.state,
                    x = _state.x,
                    y = _state.y,
                    width = _state.width,
                    height = _state.height,
                    opacity = _state.opacity,
                    visibility = _state.visibility,
                    animate = _state.animate;

                styles = getOverlayStyles(x, y, width, height, animate);

                var containerStyle = {
                    opacity: opacity,
                    transition: 'opacity ' + TRANSITION_SPEED + 's linear',
                    visibility: visibility
                };

                return _react2.default.createElement(
                    'div',
                    { style: containerStyle, 'data-selector': 'focus-overlay' },
                    _react2.default.createElement(
                        'div',
                        { style: styles.topleft, onClick: onExit },
                        this.renderChildren('topleft', children, target)
                    ),
                    _react2.default.createElement(
                        'div',
                        { style: styles.top, onClick: onExit },
                        this.renderChildren('top', children, target)
                    ),
                    _react2.default.createElement(
                        'div',
                        { style: styles.topright, onClick: onExit },
                        this.renderChildren('topright', children, target)
                    ),
                    _react2.default.createElement(
                        'div',
                        { style: styles.left, onClick: onExit },
                        this.renderChildren('left', children, target)
                    ),
                    _react2.default.createElement(
                        'div',
                        { style: styles.right, onClick: onExit },
                        this.renderChildren('right', children, target)
                    ),
                    _react2.default.createElement(
                        'div',
                        { style: styles.bottomleft, onClick: onExit },
                        this.renderChildren('bottomleft', children, target)
                    ),
                    _react2.default.createElement(
                        'div',
                        { style: styles.bottom, onClick: onExit },
                        this.renderChildren('bottom', children, target)
                    ),
                    _react2.default.createElement(
                        'div',
                        { style: styles.bottomright, onClick: onExit },
                        this.renderChildren('bottomright', children, target)
                    )
                );
            }
        }]);

        return FocusOverlay;
    }(_react2.default.Component);

    ;

    FocusOverlay.propTypes = {
        target: _propTypes2.default.oneOf(['topleft', 'top', 'topright', 'left', 'right', 'bottomleft', 'bottom', 'bottomright']),
        focusComponent: _propTypes2.default.any,
        onExit: _propTypes2.default.func,
        children: _propTypes2.default.node
    };

    FocusOverlay.defaultProps = {
        target: 'bottom'
    };

    exports.default = FocusOverlay;
});