(function (global, factory) {
    if (typeof define === "function" && define.amd) {
        define(['exports', './arrays'], factory);
    } else if (typeof exports !== "undefined") {
        factory(exports, require('./arrays'));
    } else {
        var mod = {
            exports: {}
        };
        factory(mod.exports, global.arrays);
        global.getOuterBoundries = mod.exports;
    }
})(this, function (exports, _arrays) {
    'use strict';

    Object.defineProperty(exports, "__esModule", {
        value: true
    });
    exports.getBoundriesFromFocusComponent = exports.getHighestCommonDenominator = exports.getBoundryCoordinatesFromRect = undefined;
    var getBoundryCoordinatesFromRect = exports.getBoundryCoordinatesFromRect = function getBoundryCoordinatesFromRect(rect) {
        return {
            x1: rect.x,
            x2: rect.x + rect.width,
            y1: rect.y,
            y2: rect.y + rect.height
        };
    };

    var getHighestCommonDenominator = exports.getHighestCommonDenominator = function getHighestCommonDenominator(component, current) {
        var componentBoundries = getBoundryCoordinatesFromRect(component);
        if (current == null) {
            return componentBoundries;
        }
        return {
            x1: Math.min(componentBoundries.x1, current.x1),
            x2: Math.max(componentBoundries.x2, current.x2),
            y1: Math.min(componentBoundries.y1, current.y1),
            y2: Math.max(componentBoundries.y2, current.y2)
        };
    };

    var getBoundriesFromFocusComponent = exports.getBoundriesFromFocusComponent = function getBoundriesFromFocusComponent(focusComponent) {
        if (typeof focusComponent.getBoundingClientRect === 'function') {
            return focusComponent.getBoundingClientRect();
        }
        return {
            x: focusComponent.x,
            y: focusComponent.y,
            width: focusComponent.width,
            height: focusComponent.height
        };
    };

    exports.default = function (focusComponent) {
        var result = null;

        if (focusComponent == null) {
            return {
                width: 0,
                height: 0,
                x: 0,
                y: 0
            };
        }

        if ((0, _arrays.isArray)(focusComponent)) {
            var highestCommonDenominator = void 0;
            focusComponent.map(function (component) {
                highestCommonDenominator = getHighestCommonDenominator(getBoundriesFromFocusComponent(component), highestCommonDenominator);
            });
            return {
                width: highestCommonDenominator.x2 - highestCommonDenominator.x1,
                height: highestCommonDenominator.y2 - highestCommonDenominator.y1,
                x: highestCommonDenominator.x1,
                y: highestCommonDenominator.y1
            };
        }
        return getBoundriesFromFocusComponent(focusComponent);
    };
});